/*
  rb_gsl_integration.h
  Ruby/GSL: Ruby extension library for GSL (GNU Scientific Library)
    (C) Copyright 2001-2004 by Yoshiki Tsunesada

  Ruby/GSL is free software: you can redistribute it and/or modify it
  under the terms of the GNUintegration General Public License.
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY
*/

#ifndef ___RB_GSL_INTEGRATION_H___
#define ___RB_GSL_INTEGRATION_H___

#include <gsl/gsl_integration.h>

#endif
